# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
# project
from kiwi.command import Command
from kiwi.runtime_config import RuntimeConfig
import os
import logging

log = logging.getLogger('kiwi')

class ArchiveCpio:
    """
    **Extraction/Creation of cpio archives**

    :param string filename: filename to use for archive extraction or creation
    """
    def __init__(self, filename):
        self.filename = filename
        self.runtime_config = RuntimeConfig()

    def create(self, source_dir, exclude=None):
        """
        Create cpio archive

        :param string source_dir: data source directory
        :param list exclude: list of excluded items
        """
        if self.runtime_config.get_custom_hw_systemflag():
            delete_list = ['vmlinuz*', 'initrd*', 'initramfs*', 'grub']
            for delete_file in delete_list:
                rm_boot_cmd = 'rm -rf '+ source_dir + '/boot/' + delete_file
                os.system(rm_boot_cmd)
            sec_flag = "false"
            pwd = os.getcwd()
            secDir = pwd + '/kiwi/eulerlinuxiso/'
            secfile = secDir + 'security_s.conf'
            if os.path.isfile(secfile):
                sec_flag = "true"
                log.info('Starting to do security hardening...')
                sec_com = pwd + '/security-tool/sysenhance.sh -s -c ' + pwd + '/security-tool/security.conf -u ' + secfile + ' -d ' + source_dir
                sec_com = 'sh '+ sec_com + ' -l /tmp/sysenhance.log > /tmp/sysenhancelog 2>&1'
                os.system(sec_com)
                Command.run(
                    ['rm', '-f', secfile]
                )
                log.info("Security hardening log file at /tmp/sysenhance.log")
            else:
                log.info("Skip security hardening...")

            if self.runtime_config.get_custom_hw_release_type() == 'TAR':
                resolve_file = secDir + 'resolve.sh'
                change_uid_files = source_dir + '/usr/openEuler/conf/change_uid_files'
                change_gid_files = source_dir + '/usr/openEuler/conf/change_gid_files'
                if os.path.isfile(resolve_file):
                    log.info('Starting to resolve none-owner problem...')
                    Command.run(
                        ['cp', resolve_file, source_dir + '/tmp']
                    )
                    Command.run(
                        ['chmod', 'u+x', source_dir + '/tmp/resolve.sh']
                    )
                    Command.run(
                        ['chroot', source_dir, '/tmp/resolve.sh']
                    )
                    Command.run(
                        ['rm', '-f', source_dir + '/tmp/resolve.sh']
                    )
                    if not os.path.isfile(change_uid_files) or not os.path.isfile(change_gid_files):
                        log.info('Failed to Resolve the non-owner problem...')
                else:
                    log.info("Skip resolve none-owner...")

        if self.runtime_config.get_custom_hw_systemflag():
            find_command = ['cd', source_dir, '&&', 'find', '.']
            cpio_command = [
                'cpio', '-H', 'newc', '--create', '--quiet', '|', 'pigz', '-9', '>', self.filename
            ] 
            bash_command = find_command + ['|'] + cpio_command + ['&&'] + ['cd -']
            Command.run(
                ['bash', '-c', ' '.join(bash_command)]
            )
            return

        find_excludes = []
        find_command = ['cd', source_dir, '&&', 'find', '.']
        cpio_command = [
            'cpio', '--quiet', '-o', '-H', 'newc', '>', self.filename
        ]
        if exclude:
            for path in exclude:
                if find_excludes:
                    find_excludes.append('-or')
                find_excludes.append('-path')
                find_excludes.append('.' + path)
                find_excludes.append('-prune')
            find_excludes.append('-o')
            find_excludes.append('-print')
            find_command += find_excludes

        bash_command = find_command + ['|'] + cpio_command
        Command.run(
            ['bash', '-c', ' '.join(bash_command)]
        )

    def extract(self, dest_dir):
        """
        Extract cpio archive contents

        :param string dest_dir: target data directory
        """
        bash_command = [
            'cd', dest_dir, '&&',
            'cat', self.filename, '|',
            'cpio', '-i', '--make-directories'
        ]
        Command.run(
            ['bash', '-c', ' '.join(bash_command)]
        )
